@@ -3,6 +3,8 @@
 import enum
 import attr
 
+import six
+
 
 @attr.s
 class InvalidDataLength(Exception):
@@ -34,9 +36,9 @@ class InvalidValue(Exception):
             message = value
         message = hex(value) if isinstance(value, int) else repr(value)
         type_name = type_class.__name__ if hasattr(type_class, '__name__') else str(type(type_class))
-        message = u'{} is not a valid {}'.format(message, type_name)
+        message = six.ensure_text('{} is not a valid {}').format(message, type_name)
         if class_member is not None:
-            message = u'{} {} value'.format(message, class_member)
+            message = six.ensure_text('{} {} value').format(message, class_member)
 
         super(InvalidValue, self).__init__(message)
 
