@@ -12,6 +12,8 @@ import attr
 import six
 
 from cryptoparser.common.base import (
+    OneByteEnumComposer,
+    OneByteEnumParsable,
     Opaque,
     OpaqueParam,
     VariantParsable,
@@ -82,7 +84,11 @@ class SubprotocolParser(object):
 class TlsSubprotocolMessageBase(ParsableBase):
     @classmethod
     @abc.abstractmethod
-    def get_content_type(cls):
+    def _parse(cls, parsable):
+        raise NotImplementedError()
+
+    @abc.abstractmethod
+    def compose(self):
         raise NotImplementedError()
 
 
@@ -130,10 +136,6 @@ class TlsAlertMessage(TlsSubprotocolMessageBase):
     level = attr.ib()
     description = attr.ib()
 
-    @classmethod
-    def get_content_type(cls):
-        return TlsContentType.ALERT
-
     @classmethod
     def _parse(cls, parsable):
         if len(parsable) < cls._SIZE:
@@ -180,10 +182,6 @@ class TlsChangeCipherSpecMessage(TlsSubprotocolMessageBase):
         validator=attr.validators.in_(TlsChangeCipherSpecType)
     )
 
-    @classmethod
-    def get_content_type(cls):
-        return TlsContentType.CHANGE_CIPHER_SPEC
-
     @classmethod
     def _parse(cls, parsable):
         parser = ParserBinary(parsable)
@@ -204,10 +202,6 @@ class TlsChangeCipherSpecMessage(TlsSubprotocolMessageBase):
 class TlsApplicationDataMessage(TlsSubprotocolMessageBase):
     data = attr.ib(attr.validators.instance_of(bytearray))
 
-    @classmethod
-    def get_content_type(cls):
-        return TlsContentType.APPLICATION_DATA
-
     @classmethod
     def _parse(cls, parsable):
         return TlsApplicationDataMessage(parsable), len(parsable)
@@ -241,10 +235,6 @@ class TlsHandshakeMessage(TlsSubprotocolMessageBase):
     """
     _HEADER_SIZE = 4
 
-    @classmethod
-    def get_content_type(cls):
-        return TlsContentType.HANDSHAKE
-
     @classmethod
     @abc.abstractmethod
     def get_handshake_type(cls):
@@ -396,18 +386,35 @@ class TlsCipherSuiteVector(VectorParsable):
         )
 
 
-class TlsCompressionMethod(enum.IntEnum):
-    NULL = 0
+class TlsCompressionMethodFactory(OneByteEnumParsable):
+    @classmethod
+    def get_enum_class(cls):
+        return TlsCompressionMethod
+
+    @abc.abstractmethod
+    def compose(self):
+        raise NotImplementedError()
 
 
-class TlsCompressionMethodVector(Vector):
+@attr.s(frozen=True)
+class TlsCompressionMethodParams(object):
+    code = attr.ib(validator=attr.validators.instance_of(int))
+
+
+class TlsCompressionMethod(OneByteEnumComposer, enum.Enum):
+    NULL = TlsCompressionMethodParams(code=0x00)
+    DEFLATE = TlsCompressionMethodParams(code=0x01)
+    LZS = TlsCompressionMethodParams(code=0x40)
+
+
+class TlsCompressionMethodVector(VectorParsable):
     @classmethod
     def get_param(cls):
-        return VectorParamNumeric(
-            item_size=1,
+        return VectorParamParsable(
+            item_class=TlsCompressionMethodFactory,
+            fallback_class=TlsInvalidTypeOneByte,
             min_byte_num=1,
             max_byte_num=2 ** 8 - 1,
-            numeric_class=TlsCompressionMethod
         )
 
 
@@ -564,6 +571,7 @@ class TlsHandshakeServerHello(TlsHandshakeHello):
     )
     compression_method = attr.ib(
         default=TlsCompressionMethod.NULL,
+        converter=TlsCompressionMethod,
         validator=attr.validators.in_(TlsCompressionMethod),
     )
     cipher_suite = attr.ib(default=None, validator=attr.validators.in_(TlsCipherSuite))
@@ -584,7 +592,7 @@ class TlsHandshakeServerHello(TlsHandshakeHello):
         parser = cls._parse_hello_header(handshake_header_parser['payload'])
 
         parser.parse_parsable('cipher_suite', TlsCipherSuiteFactory)
-        parser.parse_numeric('compression_method', 1, TlsCompressionMethod)
+        parser.parse_parsable('compression_method', TlsCompressionMethodFactory)
 
         extension_parser = cls._parse_extensions(handshake_header_parser, parser, TlsExtensionsServer)
 
@@ -604,7 +612,7 @@ class TlsHandshakeServerHello(TlsHandshakeHello):
         payload_composer.compose_parsable(self.random)
         payload_composer.compose_parsable(self.session_id)
         payload_composer.compose_parsable(self.cipher_suite)
-        payload_composer.compose_numeric(self.compression_method.value, 1)
+        payload_composer.compose_parsable(self.compression_method)
 
         extension_bytes = self._compose_extensions(self.extensions)
 
@@ -898,7 +906,7 @@ class TlsHandshakeHelloRetryRequest(TlsHandshakeHello):
 
         parser.parse_parsable('cipher_suite', TlsCipherSuiteFactory)
 
-        parser.parse_numeric('compression_method', 1)
+        parser.parse_parsable('compression_method', TlsCompressionMethodFactory)
         compression_method = parser['compression_method']
         session_id = parser['session_id']
 
@@ -920,7 +928,7 @@ class TlsHandshakeHelloRetryRequest(TlsHandshakeHello):
         payload_composer.compose_parsable(self.random_bytes)
         payload_composer.compose_parsable(self.session_id)
         payload_composer.compose_parsable(self.cipher_suite)
-        payload_composer.compose_numeric(self.compression_method.value, 1)
+        payload_composer.compose_parsable(self.compression_method)
 
         extension_bytes = self._compose_extensions(self.extensions)
 
